﻿module CGameController;
import std.algorithm, std.conv, std.math;
import DOpenGL.gl;
import Obelisk.CColour, Obelisk.CVector, Obelisk.IObelisk;
import CGameState;

class CGameController
{
	private IObelisk mObelisk;

	private enum ETitleState { Disabled, Scrolling, Waiting, Fading }
	private ETitleState mTitleState = ETitleState.Disabled;
	private string mTitleText;
	private int mTitleX;
	private uint mTitleWaitTimer;
	private int mTitleFade;
	private const int cMaxTitleFade = 45;
	private CGameState mGameState;

	private int mPlayerFrame = 0;
	private int mPlayerAnimCount = 20;

	private bool mShowWeaponUpgraded = false;
	private uint mShowWeaponUpgradedTimer;

	private bool mWin = false;

	public this(IObelisk aObelisk)
	{
		mObelisk = aObelisk;
		glFrontFace(GL_CW);
		mGameState = new CGameState(mObelisk);
		ShowTitle(1);
	}



	public void Update()
	{
		if (mWin)
			return;

		final switch (mTitleState)
		{
			case ETitleState.Disabled:
				break;
			case ETitleState.Scrolling:
				mTitleX -= 13;
				if (mTitleX <= 112)
				{
					mTitleX = 112;
					mTitleState = ETitleState.Waiting;
					mTitleWaitTimer = mObelisk.CreateTimer();
				}
				break;
			case ETitleState.Waiting:
				if (mObelisk.Duration(mTitleWaitTimer) > 1000)
					mTitleState = ETitleState.Fading;
				break;
			case ETitleState.Fading:
				if (++mTitleFade == cMaxTitleFade)
				{
					mTitleState = ETitleState.Disabled;
					mGameState.NextWave();
				}
				break;
		}

		if (!mGameState.GameOver())
		{
			if (mGameState.Update())
			{
				if (mGameState.GetWave() % 6 == 0)
				{
					if (mGameState.GetWave() == 6 * 6)
						mWin = true;
					else
					{
						ShowTitle(mGameState.GetWave() / 6 + 1);
						mGameState.NextLevel();
					}
				}
				else
					mGameState.NextWave();
			}

			if (mGameState.GetBulletLevelUp())
			{
				mGameState.ResetBulletLevelUp();
				mShowWeaponUpgraded = true;
				mShowWeaponUpgradedTimer = mObelisk.CreateTimer();
			}
			else if (mShowWeaponUpgraded && mObelisk.Duration(mShowWeaponUpgradedTimer) > 2000)
				mShowWeaponUpgraded = false;

			if (--mPlayerAnimCount == 0)
			{
				mPlayerAnimCount = 20;
				mPlayerFrame = 1 - mPlayerFrame;
			}
		}
	}



	public void Render()
	{
		mObelisk.SetLinearFiltering(false);
		// render background
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		mObelisk.SetSpriteViewport(700, 500);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		if (mWin)
		{
			mObelisk.SetFont("Square721 BT_22BI.bmp");
			mObelisk.DrawText("You escape from the", 57, 20);
			mObelisk.DrawText("tunnel and find a giant", 23, 70);
			mObelisk.DrawText("pile of treasure.", 113, 120);
			mObelisk.SetFont("Square721 BT_60BI.bmp");
			mObelisk.DrawText("The End", 72, 190);
			mObelisk.SetFont("Square721 BT_22BI.bmp");
			mObelisk.DrawText("Close the window", 103, 370);
			mObelisk.DrawText("to quit", 251, 420);
			return;
		}

		auto Background = new CColour(0.8078, 0.7921, 0.7921);
		auto b0 = new CVector(0, 0, 1);
		auto b1 = new CVector(700, 0, 1);
		auto b2 = new CVector(700, 195, 1);
		auto b3 = new CVector(0, 195, 1);
		mObelisk.TriColour(b0, b1, b2, Background);
		mObelisk.TriColour(b0, b2, b3, Background);
		auto Ground = new CColour(0.9176, 0.8980, 0.8980);
		auto g0 = new CVector(0, 195, 1);
		auto g1 = new CVector(700, 195, 1);
		auto g2 = new CVector(700, 500, 1);
		auto g3 = new CVector(0, 500, 1);
		mObelisk.TriColour(g0, g1, g2, Ground);
		mObelisk.TriColour(g0, g2, g3, Ground);
		int CameraX = mGameState.GetPlayerX() - 35;
		auto Stripe = new CColour(0.8784, 0.8588, 0.8588);

		// render background stripes
		glMatrixMode(GL_PROJECTION);
		glTranslatef(-CameraX, 0, 0);
		int FirstStripeX = CameraX - CameraX % 350;
		for (int i = 0; i < 4; i++)
		{
			auto s0 = new CVector(FirstStripeX + i * 350 - 305, 195, 1);
			auto s1 = new CVector(FirstStripeX + i * 350 - 305 + 80, 195, 1);
			auto s2 = new CVector(FirstStripeX + i * 350 + 80, 500, 1);
			auto s3 = new CVector(FirstStripeX + i * 350, 500, 1);
			mObelisk.TriColour(s0, s1, s2, Stripe);
			mObelisk.TriColour(s0, s2, s3, Stripe);
		}

		glScalef(1, 1, 1.0f / 512.0f);

		// render game stuff
		glMatrixMode(GL_MODELVIEW);
		glTranslatef(0, 205, 0);
		// render player
		glPushMatrix();
		glTranslatef(mGameState.GetPlayerX(), mGameState.GetPlayerY(), 511 - mGameState.GetPlayerY());
		auto PlayerSprite = mObelisk.GetTexture("MainGuy" ~ to!string(mPlayerFrame) ~ ".bmp");
		mObelisk.DrawSprite(PlayerSprite, 0, 0, -1, -1, 0.5, 0.9);
		auto GunSprite = mObelisk.GetTexture("Gun0.bmp");
		mObelisk.DrawSprite(GunSprite, 9, -26);
		glPopMatrix();
		// render robots
		auto Robots = mGameState.GetRobots();
		if (Robots !is null)
		{
			CRobot Selected = null;
			for (Robots.First(); Robots.HasNext(); Robots.Next())
			{
				if (Robots().IsSelected())
					Selected = Robots();
				glPushMatrix();
				glTranslatef(Robots().GetX(), Robots().GetY(), 511 - Robots().GetY());
				auto RobotSprite = mObelisk.GetTexture("Robot" ~ (Robots().IsDamaged() ? "Dmg" : "") ~
						to!string(Robots().GetType()) ~ ".bmp");
				mObelisk.DrawSprite(RobotSprite, 0, 0, -1, -1, 0.5, 0.9);
				glPopMatrix();
			}
			if (Selected !is null)
			{
				glPushMatrix();
				glTranslatef(Selected.GetX(), Selected.GetY(), 0);
				auto SelectSprite = mObelisk.GetTexture("Select.bmp");
				mObelisk.DrawSprite(SelectSprite, 0, 0, -1, -1, 0.5, 0.84);
				glPopMatrix();
			}
		}
		// render bullets
		auto Bullets = mGameState.GetBullets();
		for (Bullets.First(); Bullets.HasNext(); Bullets.Next())
		{
			glPushMatrix();
			glTranslatef(Bullets().GetX(), Bullets().GetY(), min(511 - Bullets().GetY(), 511));
			auto BulletSprite = mObelisk.GetTexture("Bullet" ~ to!string(mGameState.GetBulletLevel()) ~ ".bmp");
			mObelisk.DrawSprite(BulletSprite, 0, 0, -1, -1, 1, 0.5);
			glPopMatrix();
		}
		// render fireballs
		auto Fireballs = mGameState.GetFireballs();
		for (Fireballs.First(); Fireballs.HasNext(); Fireballs.Next())
		{
			glPushMatrix();
			glTranslatef(Fireballs().GetX(), Fireballs().GetY(), min(511 - Fireballs().GetY(), 511));
			if (Fireballs().GetType() == 0 && !Fireballs().IsFirst())
			{
				const float cFireballXSize = 60.0f;
				const float cFireballYSize = 11.0f;
				glRotatef(-(Fireballs().GetAngle() + PI) * (180 / PI), 0, 0, -1);
				glTranslatef(-cFireballYSize / 2, -cFireballYSize / 2, 0);
				auto F0 = new CVector(0, 0, 0);
				auto F1 = new CVector(cFireballXSize, 0, 0);
				auto F2 = new CVector(cFireballXSize, cFireballYSize, 0);
				auto F3 = new CVector(0, cFireballYSize, 0);
				mObelisk.QuadShader(F0, 0, 0, F1, 1, 0, F2, 1, 1, F3, 0, 1, "Fireball0.glsl");
			}
			else
				DrawFireball(Fireballs().GetRadius(), Fireballs().GetType());
			glPopMatrix();
		}

		// render UI
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		mObelisk.SetSpriteViewport(700, 500);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		if (mGameState.GameOver())
		{
			mObelisk.SetFont("Square721 BT_60BI.bmp");
			mObelisk.DrawText("Game", 171, 70);
			mObelisk.DrawText("Over", 192, 210);
			mObelisk.SetFont("Square721 BT_22BI.bmp");
			mObelisk.DrawText("Close the window", 103, 370);
			mObelisk.DrawText("to quit", 251, 420);
		}
		else
		{
			mObelisk.SetFont("Square721 BT_22BI.bmp");
			mObelisk.DrawText("HP: " ~ to!string(mGameState.GetPlayerHP()), 10, 0);
			mObelisk.DrawSprite(mObelisk.GetTexture("Select.bmp"), 10, 50);
			mObelisk.DrawText(to!string(mGameState.GetSelectKills()), 66, 46);
			if (mShowWeaponUpgraded)
				mObelisk.DrawText("Weapon Upgraded", 102, 110);
			if (mTitleState != ETitleState.Disabled)
			{
				mObelisk.SetFont("Square721 BT_60BI.bmp");
				mObelisk.SetAlpha(cast(float)(cMaxTitleFade - mTitleFade) / cMaxTitleFade);
				mObelisk.DrawText(mTitleText, mTitleX, 160);
				mObelisk.SetAlpha(1);
			}
		}
	}



	private void DrawFireball(int aRadius, int aType)
	{
		CColour Colour;
		final switch (aType)
		{
			case 0:
				Colour = new CColour(1, 0.8549, 0.2784);
				break;
			case 1:
				Colour = new CColour(0.95, 0.2, 0.2);
				break;
			case 2:
				Colour = new CColour(0.4902, 0.5216, 0.5294);
				break;
			case 3:
				Colour = new CColour(0.1647, 0.1529, 0.9098);
				break;
			case 4:
				Colour = new CColour(0.1255, 0.1255, 0.7490);
				break;
			case 5:
				Colour = new CColour(0.3961, 0.8706, 0.9412);
				break;
		}
		auto v0 = new CVector(0, 0, 0);
		int Segments = max(aRadius, 8);
		for (int i = 0; i < Segments; i++)
		{
			float Angle1 = PI * 2 * (cast(float)i / Segments);
			float Angle2 = PI * 2 * cast(float)((i + 1) % Segments) / Segments;
			auto v1 = new CVector(cos(Angle1) * aRadius, sin(Angle1) * aRadius, 0);
			auto v2 = new CVector(cos(Angle2) * aRadius, sin(Angle2) * aRadius, 0);
			mObelisk.TriColour(v0, v1, v2, Colour);
		}
	}



	private void ShowTitle(int aLevel)
	{
		mTitleText = "Level " ~ to!string(aLevel);
		mTitleState = ETitleState.Scrolling;
		mTitleX = 700;
		mTitleFade = 0;
	}
}