﻿module CGameState;
import std.algorithm, std.math, std.random;
import Obelisk.CCollection, Obelisk.EKey, Obelisk.IObelisk;
import CBullet, CFireball, CRobot;

class CGameState
{
	private IObelisk mObelisk;
	private int mWave = 0;
	private int mPlayerX = 35;
	private int mPlayerY = 140;
	private int mPlayerHP = 100;
	private int mSelectKills = 0;
	private CCollection!CBullet mBullets;
	private int mBulletLevel = 0;
	private bool mBulletLevelUp = false;
	private int mBulletTimer = 0;
	private CCollection!CRobot mRobots;
	private CCollection!CFireball mFireballs;

	public this(IObelisk aObelisk)
	{
		mObelisk = aObelisk;
		mBullets = new CCollection!CBullet();
		mFireballs = new CCollection!CFireball();
	}



	public bool GameOver()
	{
		return mPlayerHP <= 0;
	}



	/**
	Returns true if the current wave has completed.
	*/
	public bool Update()
	{
		mPlayerX++;
		const int cVerticalSpeed = 5;
		if (mObelisk.IsKeyDown(EKey.Up))
			mPlayerY = max(mPlayerY - cVerticalSpeed, 0);
		if (mObelisk.IsKeyDown(EKey.Down))
			mPlayerY = min(mPlayerY + cVerticalSpeed, 285);
		bool WaveComplete = false;
		for (mBullets.First(); mBullets.HasNext(); mBullets.Next())
		{
			if (mBullets().Update())
				mBullets.Delete();
			else if (mRobots !is null)
			{
				for (mRobots.First(); mRobots.HasNext(); mRobots.Next())
				{
					if (!mRobots().IsDamaged() && mBullets().GetLastX() <= mRobots().GetX() && mBullets().GetX() >= mRobots().GetX() &&
							mBullets().GetY >= mRobots().GetY() - 30 && mBullets().GetY() <= mRobots().GetY() + 5)
					{
						if (mRobots().Damage(mBullets().GetDamage()))
						{
							bool WasSelected = mRobots().IsSelected();
							mRobots.Delete();
							if (WasSelected)
							{
								SelectRobot();
								AddSelectKill();
							}
						}
						mBullets.Delete();
						break;
					}
				}
			}
		}
		if (mBulletTimer != 0)
			mBulletTimer--;
		for (mFireballs.First(); mFireballs.HasNext(); mFireballs.Next())
		{
			if (mFireballs().Update())
				mFireballs.Delete();
			else if (mFireballs().Collide(mPlayerX, mPlayerY - 24))
			{
				mPlayerHP -= mFireballs().GetDamage();
				if (mPlayerHP <= 0)
					return false;
				mFireballs.Delete();
			}
		}
		if (mRobots !is null)
		{
			if (mRobots.GetCount() != 0)
			{
				if (mBulletTimer == 0)
				{
					mBullets.Add(new CBullet(mPlayerX + 40, mPlayerY - 25, mBulletLevel + 2, 46));
					mBulletTimer = 20;
				}
			}
			for (mRobots.First(); mRobots.HasNext(); mRobots.Next())
			{
				mRobots().Update();
				if (mRobots().GetX() < mPlayerX - 50)
				{
					bool WasSelected = mRobots().IsSelected();
					mRobots.Delete();
					if (WasSelected)
						SelectRobot();
				}
				else if (mRobots().CanShoot())
				{
					mRobots().Shoot();
					int X = mRobots().GetX() - 15;
					int Y = mRobots().GetY() - 23;
					mFireballs.Add(new CFireball(X, Y, AngleToPlayer(X, Y, mRobots().GetFireballRadius()),
							mRobots().GetType(), mRobots().GetFireballDamage(), mRobots().GetFireballSpeed(),
							mRobots().GetFireballRadius()));
				}
			}
			if (mRobots.GetCount() == 0)
			{
				WaveComplete = true;
				mWave++;
				mRobots = null;
			}
		}
		return WaveComplete;
	}



	private float AngleToPlayer(int aX, int aY, int aRadius)
	{
		return atan2(cast(float)(mPlayerY - 24 + uniform!"[]"(-aRadius, aRadius) - aY),
				cast(float)(mPlayerX - aX));
	}



	public int GetWave()
	{
		return mWave;
	}



	public int GetPlayerX()
	{
		return mPlayerX;
	}



	public int GetPlayerY()
	{
		return mPlayerY;
	}



	public int GetPlayerHP()
	{
		return mPlayerHP;
	}



	public int GetSelectKills()
	{
		return mSelectKills;
	}



	public int GetBulletLevel()
	{
		return mBulletLevel;
	}



	public bool GetBulletLevelUp()
	{
		return mBulletLevelUp;
	}



	public void ResetBulletLevelUp()
	{
		mBulletLevelUp = false;
	}



	public CCollection!CBullet GetBullets()
	{
		return mBullets;
	}



	public CCollection!CRobot GetRobots()
	{
		return mRobots;
	}



	public CCollection!CFireball GetFireballs()
	{
		return mFireballs;
	}



	public void NextWave()
	{
		switch (mWave)
		{
			// level 1
			case 0:
				SetWave([ 4, 1 ]);   // 5
				break;
			case 1:
				SetWave([ 4, 3 ]);   // 7
				break;
			case 2:
				SetWave([ 6, 0, 1 ]);   // 7
				break;
			case 3:
				SetWave([ 4, 2, 2 ]);   // 8
				break;
			case 4:
				SetWave([ 3, 3, 3 ]);   // 9
				break;
			case 5:
				SetWave([ 2, 3, 4 ]);   // 9
				break;

			// level 2
			case 6:
				SetWave([ 10 ]);   // 10
				break;
			case 7:
				SetWave([ 6, 2, 2 ]);   // 10
				break;
			case 8:
				SetWave([ 6, 0, 4 ]);   // 10
				break;
			case 9:
				SetWave([ 10, 4 ]);   // 14
				break;
			case 10:
				SetWave([ 10, 1, 1, 1 ]);
				break;
			case 11:
				SetWave([ 4, 3, 1, 2 ]);
				break;

			// level 3
			case 12:
				SetWave([ 5, 3, 3 ]);
				break;
			case 13:
				SetWave([ 4, 2, 0, 3]);
				break;
			case 14:
				SetWave([ 2, 4, 4 ]);
				break;
			case 15:
				SetWave([ 0, 10 ]);
				break;
			case 16:
				SetWave([ 0, 4, 8, 1 ]);
				break;
			case 17:
				SetWave([ 4, 4, 4, 1, 1 ]);
				break;

			// level 4
			case 18:
				SetWave([ 4, 4, 4, 1 ]);
				break;
			case 19:
				SetWave([ 0, 8, 0, 2 ]);
				break;
			case 20:
				SetWave([ 8, 4, 4, 0, 1 ]);
				break;
			case 21:
				SetWave([ 5, 7, 7 ]);
				break;
			case 22:
				SetWave([ 0, 10, 6, 2 ]);
				break;
			case 23:
				SetWave([ 4, 4, 2, 2, 2 ]);
				break;

			// level 5
			case 24:
				SetWave([ 0, 7, 7 ]);
				break;
			case 25:
				SetWave([ 0, 0, 20 ]);
				break;
			case 26:
				SetWave([ 6, 3, 3, 4 ]);
				break;
			case 27:
				SetWave([ 7, 5, 5, 0, 1 ]);
				break;
			case 28:
				SetWave([ 4, 4, 4, 2, 2 ]);
				break;
			case 29:
				SetWave([ 2, 5, 5, 2, 0, 1 ]);
				break;

			// level 6
			case 30:
				SetWave([ 0, 4, 4, 4 ]);
				break;
			case 31:
				SetWave([ 4, 8, 8, 0, 0, 1 ]);
				break;
			case 32:
				SetWave([ 0, 10, 0, 0, 2 ]);
				break;
			case 33:
				SetWave([ 0, 4, 4, 4, 2 ]);
				break;
			case 34:
				SetWave([ 0, 0, 0, 2, 2, 2 ]);
				break;
			case 35:
				SetWave([ 0, 0, 0, 3, 3, 3 ]);
				break;

			default:
				SetWave([ 6, 6, 6, 6, 6, 6 ]);
				break;
		}

		SelectRobot();
	}



	private void SelectRobot()
	{
		if (mRobots.GetCount() != 0)
		{
			int Selected = uniform(0, mRobots.GetCount());
			for (mRobots.First(); mRobots.HasNext(); mRobots.Next())
			{
				if (Selected == 0)
				{
					mRobots().Select();
					break;
				}
				Selected--;
			}
		}
	}



	private void SetWave(int[] aWaveInfo)
	{
		mRobots = new CCollection!CRobot();
		foreach (int Type, int Count; aWaveInfo)
		{
			for (int i = 0; i < Count; i++)
			{
				int x = mPlayerX + 700 + uniform(0, 90);
				int y = uniform(0, 285);
				int HP;
				int Damage;
				int FireRate;
				int Speed;
				int Radius;
				final switch (Type)
				{
					case 0:
						HP = 2;
						Damage = 1;
						FireRate = 16;
						Speed = 57;
						Radius = 3;
						break;
					case 1:
						HP = 3;
						Damage = 4;
						FireRate = 30;
						Speed = 8;
						Radius = 7;
						break;
					case 2:
						HP = 4;
						Damage = 5;
						FireRate = 40;
						Speed = 14;
						Radius = 5;
						break;
					case 3:
						HP = 6;
						Damage = 15;
						FireRate = 80;
						Speed = 10;
						Radius = 16;
						break;
					case 4:
						HP = 7;
						Damage = 20;
						FireRate = 28;
						Speed = 7;
						Radius = 22;
						break;
					case 5:
						HP = 12;
						Damage = 34;
						FireRate = 120;
						Speed = 18;
						Radius = 33;
						break;
				}
				mRobots.Add(new CRobot(x, y, Type, HP, Damage, FireRate, Speed, Radius));
			}
		}
	}



	public void NextLevel()
	{
		mPlayerHP = 100 + mWave / 6 * 10;
	}



	private void AddSelectKill()
	{
		mSelectKills++;
		final switch (mBulletLevel)
		{
			case 0:
				if (mSelectKills == 30)
					mBulletLevelUp = true;
				break;
			case 1:
				if (mSelectKills == 60)
					mBulletLevelUp = true;
				break;
			case 2:
				if (mSelectKills == 90)
					mBulletLevelUp = true;
				break;
			case 3:
				if (mSelectKills == 120)
					mBulletLevelUp = true;
				break;
			case 4:
				break;
		}
		if (mBulletLevelUp)
			mBulletLevel++;
	}
}