﻿module CMainController;
import std.conv;
import Obelisk.EKey, Obelisk.IMainController, Obelisk.IObelisk, Obelisk.IThread;
import CGameController;

extern (C) IMainController CreateMainController(IObelisk aObelisk)
{
	return new CMainController(aObelisk);
}



class CMainController : IMainController
{
	private IObelisk mObelisk;
	private CGameController mGameController;
	private bool mPaused = false;

	public this(IObelisk aObelisk)
	{
		mObelisk = aObelisk;
		mObelisk.SetSize(700, 500);
		mObelisk.SetCaption("Robot Tunnel");
	}



	public void Initialise()
	{
		mObelisk.LoadTexture("MainGuy0.bmp", true);
		mObelisk.LoadTexture("MainGuy1.bmp", true);
		mObelisk.LoadTexture("Gun0.bmp", true);
		for (int i = 0; i <= 4; i++)
			mObelisk.LoadTexture("Bullet" ~ to!string(i) ~ ".bmp", false);
		for (int i = 0; i <= 5; i++)
		{
			mObelisk.LoadTexture("Robot" ~ to!string(i) ~ ".bmp", true);
			mObelisk.LoadTexture("RobotDmg" ~ to!string(i) ~ ".bmp", true);
		}
		mObelisk.LoadTexture("Select.bmp", true);
		mObelisk.LoadFontImage("Square721 BT_60BI.bmp");
		mObelisk.LoadFontImage("Square721 BT_28BI.bmp");
		mObelisk.LoadFontImage("Square721 BT_22BI.bmp");
		mObelisk.LoadShader("Fireball0.glsl");
		mObelisk.StartTimer();
	}



	public void OnNotify(int aMessage, int aData)
	{
	}



	public void OnTimer()
	{
		if (mGameController is null)
		{
			if (mObelisk.IsKeyPressed(EKey.Enter))
				mGameController = new CGameController(mObelisk);
		}
		else
		{
			if (!mPaused)
				mGameController.Update();
		}
		mObelisk.BeginFrame();
		if (mGameController is null)
		{
			mObelisk.SetFont("Square721 BT_28BI.bmp");
			mObelisk.DrawText("Robot Tunnel", 140, 30);

			mObelisk.SetFont("Square721 BT_22BI.bmp");
			mObelisk.DrawText("You must escape", 109, 120);
			mObelisk.DrawText("from a tunnel filled", 69, 170);
			mObelisk.DrawText("with evil robots.", 112, 220);
			mObelisk.DrawText("Up and Down to move.", 39, 300);
			mObelisk.DrawText("Shooting is automatic.", 26, 350);
			mObelisk.DrawText("(Press Enter to start)", 25, 420);
		}
		else
			mGameController.Render();
		mObelisk.EndFrame();
	}



	public void OnDeactivate()
	{
		mPaused = true;
	}



	public void OnActivate()
	{
		mPaused = false;
	}



	public void OnKeyDown(EKey aKey)
	{
	}



	public void OnKeyUp(EKey aKey)
	{
	}



	public void ThreadFinished(IThread aThread)
	{
	}
}